@@ -52,6 +52,7 @@
 #define OPP_CHANNEL	9
 #define FTP_CHANNEL	10
 #define PBAP_CHANNEL	15
+#define PCSUITE_CHANNEL	24
 
 #define DEFAULT_ROOT_PATH "/tmp"
 
@@ -59,6 +60,13 @@
 
 static GMainLoop *main_loop = NULL;
 
+static int services = 0;
+static gboolean tty_needs_reinit = FALSE;
+static int signal_pipe[2];
+
+#define TTY_RX_MTU 65535
+#define TTY_TX_MTU 65535
+
 int tty_init(int services, const gchar *root_path,
 		const gchar *capability, const gchar *devnode)
 {
@@ -67,6 +75,8 @@ int tty_init(int services, const gchar *root_path,
 	int fd, ret;
 	glong flags;
 
+	tty_needs_reinit = TRUE;
+
 	fd = open(devnode, O_RDWR);
 	if (fd < 0)
 		return fd;
@@ -78,18 +88,21 @@ int tty_init(int services, const gchar *root_path,
 	cfmakeraw(&options);
 	tcsetattr(fd, TCSANOW, &options);
 
-	server = g_malloc0(sizeof(struct server));
+	server = g_new0(struct server, 1);
 	server->services = services;
 	server->folder = g_strdup(root_path);
 	server->auto_accept = TRUE;
 	server->capability = g_strdup(capability);
 	server->devnode = g_strdup(devnode);
+	server->rx_mtu = TTY_RX_MTU;
+	server->tx_mtu = TTY_TX_MTU;
 
 	ret = obex_session_start(fd, server);
 	if (ret < 0) {
 		server_free(server);
 		close(fd);
-	}
+	} else
+		tty_needs_reinit = FALSE;
 
 	return ret;
 }
@@ -110,6 +123,7 @@ static gboolean option_autoaccept = FALSE;
 static gboolean option_opp = FALSE;
 static gboolean option_ftp = FALSE;
 static gboolean option_pbap = FALSE;
+static gboolean option_pcsuite = FALSE;
 
 static GOptionEntry options[] = {
 	{ "nodaemon", 'n', G_OPTION_FLAG_REVERSE,
@@ -131,15 +145,60 @@ static GOptionEntry options[] = {
 				"Enable File Transfer server" },
 	{ "pbap", 'p', 0, G_OPTION_ARG_NONE, &option_pbap,
 				"Enable Phonebook Access server" },
+	{ "pcsuite", 's', 0, G_OPTION_ARG_NONE, &option_pcsuite,
+				"Enable PC Suite Services server" },
 	{ NULL },
 };
 
+static void sig_usr1(int sig)
+{
+	if (write(signal_pipe[1], &sig, sizeof(sig)) != sizeof(sig))
+		error("unable to write to signal pipe");
+}
+
+static gboolean handle_signal(GIOChannel *io, GIOCondition cond,
+				void *user_data)
+{
+	int sig, fd = g_io_channel_unix_get_fd(io);
+
+	if (read(fd, &sig, sizeof(sig)) != sizeof(sig)) {
+		error("handle_sigusr1: unable to read signal from pipe");
+		return TRUE;
+	}
+
+	if (sig == SIGUSR1 && tty_needs_reinit)
+		tty_init(services, option_root, option_capability,
+				option_devnode);
+
+	return TRUE;
+}
+
+static int devnode_setup(void)
+{
+	struct sigaction sa;
+	GIOChannel *pipe_io;
+
+	if (pipe(signal_pipe) < 0)
+		return -errno;
+
+	pipe_io = g_io_channel_unix_new(signal_pipe[0]);
+	g_io_add_watch(pipe_io, G_IO_IN, handle_signal, NULL);
+	g_io_channel_unref(pipe_io);
+
+	memset(&sa, 0, sizeof(sa));
+	sa.sa_handler = sig_usr1;
+	sigaction(SIGUSR1, &sa, NULL);
+
+	return tty_init(services, option_root, option_capability,
+			option_devnode);
+}
+
 int main(int argc, char *argv[])
 {
 	GOptionContext *context;
 	GError *err = NULL;
 	struct sigaction sa;
-	int log_option = LOG_NDELAY | LOG_PID, services = 0;
+	int log_option = LOG_NDELAY | LOG_PID;
 
 #ifdef NEED_THREADS
 	if (g_thread_supported() == FALSE)
@@ -223,9 +282,15 @@ int main(int argc, char *argv[])
 				PBAP_CHANNEL, TRUE, FALSE, NULL);
 	}
 
+	if (option_pcsuite == TRUE) {
+		services |= OBEX_PCSUITE;
+		bluetooth_init(OBEX_PCSUITE, "Nokia OBEX PC Suite Services",
+				option_root, PCSUITE_CHANNEL, TRUE,
+				option_autoaccept, option_capability);
+	}
+
 	if (option_devnode)
-		tty_init(services, option_root, option_capability,
-				option_devnode);
+		devnode_setup();
 
 	memset(&sa, 0, sizeof(sa));
 	sa.sa_handler = sig_term;
